@@ -1,4 +1,5 @@
 use chrono::{DateTime, Duration, NaiveDate, Utc};
+use enum_dispatch::enum_dispatch;
 use num_rational::Rational64;
 use num_traits::{Signed, ToPrimitive, Zero};
 use ordered_float::NotNan;
@@ -8,45 +9,182 @@ use std::error::Error;
 use std::fmt;
 use std::str::FromStr;
 
+#[enum_dispatch]
 #[derive(Clone, Debug, Eq, PartialEq)]
-pub struct OptionPosition {
+pub enum Position {
+    OptionsPosition,
+    SharesPosition,
+}
+
+/// Defines methods that are shared between option and share positions.
+#[enum_dispatch(Position)]
+pub trait GenericPosition {
+    /// For an option position, the symbol of the option itself. For a share position, equal to [`Self::underlying_symbol()`].
+    fn symbol(&self) -> &str;
+
+    /// For an option position, the symbol of the instrument that the option is a derivative of.
+    /// For a share position, the symbol of the stock.
+    fn underlying_symbol(&self) -> &str;
+
+    /// Whether the position is long or short the underlying.
+    fn is_long(&self) -> bool;
+
+    /// The original cost per option contract or share in this position. If the position is long, this should be negative.
+    fn unit_cost(&self) -> Option<Rational64>;
+    fn unit_cost_mut(&mut self) -> &mut Option<Rational64>;
+
+    /// The current bid price per option contract or share in this position. If the position is long, this should be positive.
+    fn unit_bid_price(&self) -> Option<Rational64>;
+    fn unit_bid_price_mut(&mut self) -> &mut Option<Rational64>;
+
+    /// The current ask price per option contract or share in this position. If the position is long, this should be positive.
+    fn unit_ask_price(&self) -> Option<Rational64>;
+    fn unit_ask_price_mut(&mut self) -> &mut Option<Rational64>;
+
+    /// The delta per option contract or share in this position.
+    fn unit_delta(&self) -> Option<NotNan<f64>>;
+
+    /// The vega per option contract or share in this position.
+    fn unit_vega(&self) -> Option<NotNan<f64>>;
+
+    /// The theta per option contract in this position.
+    fn unit_theta(&self) -> Option<NotNan<f64>>;
+
+    /// The number of option contracts or shares in this position.
+    fn quantity(&self) -> usize;
+    fn quantity_mut(&mut self) -> &mut usize;
+
+    /// Equal to [`Self::quantity()`], but negative if the position is short.
+    fn signed_quantity(&self) -> i64 {
+        let q: i64 = self.quantity().try_into().unwrap();
+        if self.is_long() {
+            q
+        } else {
+            -q
+        }
+    }
+
+    /// The total original cost of all option contracts or shares in this position.
+    fn cost(&self) -> Option<Rational64> {
+        let q: i64 = self.quantity().try_into().unwrap();
+        self.unit_cost().map(|x| x * q)
+    }
+
+    fn net_liq(&self) -> Option<Rational64> {
+        let q: i64 = self.quantity().try_into().unwrap();
+        self.unit_mid_price().map(|x| x * q)
+    }
+
+    /// The total current bid price for all option contracts or shares in this position.
+    fn bid_price(&self) -> Option<Rational64> {
+        let q: i64 = self.quantity().try_into().unwrap();
+        self.unit_bid_price().map(|x| x * q)
+    }
+
+    /// The total current ask price for all option contracts or shares in this position.
+    fn ask_price(&self) -> Option<Rational64> {
+        let q: i64 = self.quantity().try_into().unwrap();
+        self.unit_ask_price().map(|x| x * q)
+    }
+
+    /// The total current mid price for all option contracts or shares in this position.
+    fn mid_price(&self) -> Option<Rational64> {
+        let q: i64 = self.quantity().try_into().unwrap();
+        self.unit_mid_price().map(|x| x * q)
+    }
+
+    /// The current mid price per option contract or share in this position. If the position is long, this will be positive.
+    fn unit_mid_price(&self) -> Option<Rational64> {
+        Some((self.unit_bid_price()? + self.unit_ask_price()?) / 2)
+    }
+
+    /// The total delta for all option contracts or shares in this position.
+    fn delta(&self) -> Option<NotNan<f64>> {
+        self.unit_delta().map(|x| x * self.quantity() as f64)
+    }
+
+    /// The total vega for all option contracts or shares in this position.
+    fn vega(&self) -> Option<NotNan<f64>> {
+        self.unit_vega().map(|x| x * self.quantity() as f64)
+    }
+
+    /// The total theta for all option contracts or shares in this position.
+    fn theta(&self) -> Option<NotNan<f64>> {
+        self.unit_theta().map(|x| x * self.quantity() as f64)
+    }
+
+    /// For an option position, the strike price of the option. For a share position, the strike
+    /// price of the call option with the equivalent cost at the time of purchase i.e. $0.
+    fn equivalent_strike_price(&self) -> Rational64;
+
+    /// For an option position, the type of the option. For a share position, equal to [`OptionType::Call`].
+    fn equivalent_option_type(&self) -> OptionType;
+
+    /// For an option position, the number of units of the underlying per option contract. For a
+    /// share position, equal to 1.
+    fn equivalent_lot_size(&self) -> usize;
+
+    fn profit_at_expiry(&self, underlying_price: Rational64) -> Rational64 {
+        let lot_size: i64 = self.equivalent_lot_size().try_into().unwrap();
+
+        let unit_expiry_net_liq = match self.equivalent_option_type() {
+            OptionType::Call => underlying_price - self.equivalent_strike_price(),
+            OptionType::Put => self.equivalent_strike_price() - underlying_price,
+        }
+        .max(Rational64::zero())
+            * if self.is_long() { lot_size } else { -lot_size };
+
+        let q: i64 = self.quantity().try_into().unwrap();
+        (self.unit_cost().expect("Undefined cost") + unit_expiry_net_liq) * q
+    }
+}
+
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub struct OptionsPosition {
     /// The symbol of the option itself.
     pub symbol: String,
 
     /// The symbol of the instrument that the option is a derivative of.
     pub underlying_symbol: String,
 
-    pub option_type: OptionType,
-    pub strike_price: Rational64,
-    pub expiration_date: ExpirationDate,
+    /// Whether the position is long or short the underlying.
     pub is_long: bool,
 
-    /// The original cost per contract in this position. If the position is long, this should be negative.
+    /// The original cost per option contract in this position. If the position is long, this should be negative.
     pub unit_cost: Option<Rational64>,
 
-    /// The current bid price per contract in this position. If the position is long, this should be positive.
+    /// The current bid price per option contract in this position. If the position is long, this should be positive.
     pub unit_bid_price: Option<Rational64>,
 
-    /// The current ask price per contract in this position. If the position is long, this should be positive.
+    /// The current ask price per option contract in this position. If the position is long, this should be positive.
     pub unit_ask_price: Option<Rational64>,
 
-    /// The delta per contract in this position.
+    /// The delta per option contract in this position.
     pub unit_delta: Option<NotNan<f64>>,
 
-    /// The vega per contract in this position.
+    /// The vega per option contract in this position.
     pub unit_vega: Option<NotNan<f64>>,
 
-    /// The theta per contract in this position.
+    /// The theta per option contract in this position.
     pub unit_theta: Option<NotNan<f64>>,
 
-    /// The number of contracts in this position.
+    /// The number of option contracts in this position.
     pub quantity: usize,
 
-    /// The lot size per contract. Defaults to 100 if not defined.
+    /// The strike price of the option.
+    pub strike_price: Rational64,
+
+    /// The type of the option.
+    pub option_type: OptionType,
+
+    /// The expiration date of the option.
+    pub expiration_date: ExpirationDate,
+
+    /// The number of units of the underlying per option contract in this position. Assumed to be 100 if not defined.
     pub lot_size: Option<usize>,
 }
 
-impl OptionPosition {
+impl OptionsPosition {
     pub fn description(&self) -> String {
         format!(
             "{} {:.2} {:?}",
@@ -56,81 +194,22 @@ impl OptionPosition {
         )
     }
 
-    pub fn signed_quantity(&self) -> i64 {
-        let q: i64 = self.quantity.try_into().unwrap();
-        if self.is_long {
-            q
-        } else {
-            -q
-        }
-    }
-
-    pub fn cost(&self) -> Option<Rational64> {
-        let q: i64 = self.quantity.try_into().unwrap();
-        self.unit_cost.map(|x| x * q)
-    }
-
-    pub fn net_liq(&self) -> Option<Rational64> {
-        let q: i64 = self.quantity.try_into().unwrap();
-        self.unit_mid_price().map(|x| x * q)
-    }
-
-    pub fn bid_price(&self) -> Option<Rational64> {
-        let q: i64 = self.quantity.try_into().unwrap();
-        self.unit_bid_price.map(|x| x * q)
-    }
-
-    pub fn ask_price(&self) -> Option<Rational64> {
-        let q: i64 = self.quantity.try_into().unwrap();
-        self.unit_ask_price.map(|x| x * q)
-    }
-
-    pub fn mid_price(&self) -> Option<Rational64> {
-        let q: i64 = self.quantity.try_into().unwrap();
-        self.unit_mid_price().map(|x| x * q)
-    }
-
-    pub fn unit_mid_price(&self) -> Option<Rational64> {
-        Some((self.unit_bid_price? + self.unit_ask_price?) / 2)
-    }
-
-    pub fn delta(&self) -> Option<NotNan<f64>> {
-        self.unit_delta.map(|x| x * self.quantity as f64)
-    }
-
-    pub fn vega(&self) -> Option<NotNan<f64>> {
-        self.unit_vega.map(|x| x * self.quantity as f64)
-    }
-
-    pub fn theta(&self) -> Option<NotNan<f64>> {
-        self.unit_theta.map(|x| x * self.quantity as f64)
-    }
-
-    pub fn profit_at_expiry(&self, underlying_price: Rational64) -> Rational64 {
-        let lot_size: i64 = self.lot_size.unwrap_or(100).try_into().unwrap();
-
-        let unit_expiry_net_liq = match self.option_type {
-            OptionType::Call => underlying_price - self.strike_price,
-            OptionType::Put => self.strike_price - underlying_price,
-        }
-        .max(Rational64::zero())
-            * if self.is_long { lot_size } else { -lot_size };
-
-        let q: i64 = self.quantity.try_into().unwrap();
-        (self.unit_cost.expect("Undefined cost") + unit_expiry_net_liq) * q
-    }
-
     #[cfg(test)]
-    pub fn mock(option_type: OptionType, strike_price: i64, cost: i64, quantity: usize) -> Self {
-        let is_long = cost < 0;
-        OptionPosition {
+    pub fn mock(
+        option_type: OptionType,
+        strike_price: i64,
+        unit_cost: i64,
+        quantity: usize,
+    ) -> Position {
+        let is_long = unit_cost < 0;
+        OptionsPosition {
             symbol: "OPTION".to_string(),
             underlying_symbol: "ABC".to_string(),
             option_type,
             strike_price: Rational64::from_integer(strike_price),
             expiration_date: Default::default(),
             is_long,
-            unit_cost: Some(Rational64::from_integer(cost)),
+            unit_cost: Some(Rational64::from_integer(unit_cost)),
             unit_bid_price: None,
             unit_ask_price: None,
             unit_delta: None,
@@ -139,6 +218,184 @@ impl OptionPosition {
             quantity,
             lot_size: None,
         }
+        .into()
+    }
+}
+
+impl GenericPosition for OptionsPosition {
+    fn symbol(&self) -> &str {
+        &self.symbol
+    }
+
+    fn underlying_symbol(&self) -> &str {
+        &self.underlying_symbol
+    }
+
+    fn is_long(&self) -> bool {
+        self.is_long
+    }
+
+    fn unit_cost(&self) -> Option<Rational64> {
+        self.unit_cost
+    }
+
+    fn unit_cost_mut(&mut self) -> &mut Option<Rational64> {
+        &mut self.unit_cost
+    }
+
+    fn unit_bid_price(&self) -> Option<Rational64> {
+        self.unit_bid_price
+    }
+
+    fn unit_bid_price_mut(&mut self) -> &mut Option<Rational64> {
+        &mut self.unit_bid_price
+    }
+
+    fn unit_ask_price(&self) -> Option<Rational64> {
+        self.unit_ask_price
+    }
+
+    fn unit_ask_price_mut(&mut self) -> &mut Option<Rational64> {
+        &mut self.unit_ask_price
+    }
+
+    fn unit_delta(&self) -> Option<NotNan<f64>> {
+        self.unit_delta
+    }
+
+    fn unit_vega(&self) -> Option<NotNan<f64>> {
+        self.unit_vega
+    }
+
+    fn unit_theta(&self) -> Option<NotNan<f64>> {
+        self.unit_theta
+    }
+
+    fn quantity(&self) -> usize {
+        self.quantity
+    }
+
+    fn quantity_mut(&mut self) -> &mut usize {
+        &mut self.quantity
+    }
+
+    fn equivalent_strike_price(&self) -> Rational64 {
+        self.strike_price
+    }
+
+    fn equivalent_option_type(&self) -> OptionType {
+        self.option_type
+    }
+
+    fn equivalent_lot_size(&self) -> usize {
+        self.lot_size.unwrap_or(100)
+    }
+}
+
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub struct SharesPosition {
+    /// The symbol of the stock.
+    pub symbol: String,
+
+    /// Whether the position is long or short the underlying.
+    pub is_long: bool,
+
+    /// The original cost per share in this position. If the position is long, this should be negative.
+    pub unit_cost: Option<Rational64>,
+
+    /// The current bid price per share in this position. If the position is long, this should be positive.
+    pub unit_bid_price: Option<Rational64>,
+
+    /// The current ask price per share in this position. If the position is long, this should be positive.
+    pub unit_ask_price: Option<Rational64>,
+
+    /// The number of shares in this position.
+    pub quantity: usize,
+}
+
+impl SharesPosition {
+    #[cfg(test)]
+    pub fn mock(unit_cost: i64, quantity: usize) -> Position {
+        let is_long = unit_cost < 0;
+        SharesPosition {
+            symbol: "ABC".to_string(),
+            is_long,
+            unit_cost: Some(Rational64::from_integer(unit_cost)),
+            unit_bid_price: None,
+            unit_ask_price: None,
+            quantity,
+        }
+        .into()
+    }
+}
+
+impl GenericPosition for SharesPosition {
+    fn symbol(&self) -> &str {
+        &self.symbol
+    }
+
+    fn underlying_symbol(&self) -> &str {
+        &self.symbol
+    }
+
+    fn is_long(&self) -> bool {
+        self.is_long
+    }
+
+    fn unit_cost(&self) -> Option<Rational64> {
+        self.unit_cost
+    }
+
+    fn unit_cost_mut(&mut self) -> &mut Option<Rational64> {
+        &mut self.unit_cost
+    }
+
+    fn unit_bid_price(&self) -> Option<Rational64> {
+        self.unit_bid_price
+    }
+
+    fn unit_bid_price_mut(&mut self) -> &mut Option<Rational64> {
+        &mut self.unit_bid_price
+    }
+
+    fn unit_ask_price(&self) -> Option<Rational64> {
+        self.unit_ask_price
+    }
+
+    fn unit_ask_price_mut(&mut self) -> &mut Option<Rational64> {
+        &mut self.unit_ask_price
+    }
+
+    fn unit_delta(&self) -> Option<NotNan<f64>> {
+        Some(NotNan::new(if self.is_long { 1.0 } else { -1.0 }).unwrap())
+    }
+
+    fn unit_vega(&self) -> Option<NotNan<f64>> {
+        Some(NotNan::new(0.0).unwrap())
+    }
+
+    fn unit_theta(&self) -> Option<NotNan<f64>> {
+        Some(NotNan::new(0.0).unwrap())
+    }
+
+    fn quantity(&self) -> usize {
+        self.quantity
+    }
+
+    fn quantity_mut(&mut self) -> &mut usize {
+        &mut self.quantity
+    }
+
+    fn equivalent_strike_price(&self) -> Rational64 {
+        Rational64::zero()
+    }
+
+    fn equivalent_option_type(&self) -> OptionType {
+        OptionType::Call
+    }
+
+    fn equivalent_lot_size(&self) -> usize {
+        1
     }
 }
 
@@ -183,9 +440,8 @@ impl FromStr for Decimal {
                 without_decimal
                     .len()
                     .checked_sub(decimal_idx)
-                    .ok_or_else(|| DecimalFromStrError(s.to_string()))?
-                    .try_into()
-                    .unwrap(),
+                    .and_then(|d| TryInto::<u32>::try_into(d).ok())
+                    .ok_or_else(|| DecimalFromStrError(s.to_string()))?,
             )
         } else {
             1
@@ -290,35 +546,41 @@ pub struct Breakeven {
 }
 
 // options should be sorted by strike price
-pub fn calculate_breakevens_for_strategy(options: &[OptionPosition]) -> StrategyBreakevens {
-    if options.is_empty() {
+pub fn calculate_breakevens_for_strategy(positions: &[Position]) -> StrategyBreakevens {
+    if positions.is_empty() {
         return StrategyBreakevens { breakevens: vec![] };
     }
 
-    let max_strike_price = options
+    let max_strike_price = positions
         .iter()
-        .map(|option| option.strike_price)
+        .map(|position| position.equivalent_strike_price())
         .max()
         .unwrap();
 
     let profit_at_price = |price| {
-        let profit: Rational64 = options
+        let profit: Rational64 = positions
             .iter()
-            .map(|option| option.profit_at_expiry(price))
+            .map(|position| position.profit_at_expiry(price))
             .sum();
         profit
     };
 
-    let price_range = (Rational64::zero(), max_strike_price * 100);
+    // arbitrary scale factor that once applied to the max strike price should exceed the upper breakeven
+    const MARGIN_SCALE_FACTOR: i64 = 1000;
+    let price_range = (
+        Rational64::zero(),
+        // increment by 1 to handle strike price of zero
+        (max_strike_price + 1) * MARGIN_SCALE_FACTOR,
+    );
 
     let mut prev_price = price_range.0;
     let mut prev_profit = profit_at_price(prev_price);
 
     let mut breakevens = vec![];
 
-    for strike_price in options
+    for strike_price in positions
         .iter()
-        .map(|option| option.strike_price)
+        .map(|position| position.equivalent_strike_price())
         .chain(std::iter::once(price_range.1))
     {
         if strike_price == prev_price {
@@ -385,8 +647,8 @@ impl ProfitBound {
     }
 }
 
-pub fn calculate_profit_bounds_for_strategy(options: &[OptionPosition]) -> StrategyProfitBounds {
-    if options.is_empty() {
+pub fn calculate_profit_bounds_for_strategy(positions: &[Position]) -> StrategyProfitBounds {
+    if positions.is_empty() {
         return StrategyProfitBounds {
             max_loss: None,
             max_profit: None,
@@ -395,21 +657,32 @@ pub fn calculate_profit_bounds_for_strategy(options: &[OptionPosition]) -> Strat
 
     let mut min_gradient: i64 = 0;
     let mut max_gradient: i64 = 0;
-    for option in options {
-        match (option.option_type, option.is_long) {
-            (OptionType::Call, true) => max_gradient += option.quantity as i64,
-            (OptionType::Call, false) => max_gradient -= option.quantity as i64,
-            (OptionType::Put, true) => min_gradient += option.quantity as i64,
-            (OptionType::Put, false) => min_gradient -= option.quantity as i64,
+    for position in positions {
+        let gradient_delta: i64 = (position.quantity() * position.equivalent_lot_size())
+            .try_into()
+            .unwrap();
+        match (position.equivalent_option_type(), position.is_long()) {
+            (OptionType::Call, true) => {
+                max_gradient += gradient_delta;
+            }
+            (OptionType::Call, false) => {
+                max_gradient -= gradient_delta;
+            }
+            (OptionType::Put, true) => {
+                min_gradient += gradient_delta;
+            }
+            (OptionType::Put, false) => {
+                min_gradient -= gradient_delta;
+            }
         }
     }
 
     let max_loss_at_strike = {
         let mut max_loss = Rational64::from_integer(i64::MAX);
         let mut max_loss_price = Rational64::zero();
-        for option in options {
-            let price = option.strike_price;
-            let profit_at_price = options.iter().map(|o| o.profit_at_expiry(price)).sum();
+        for position in positions {
+            let price = position.equivalent_strike_price();
+            let profit_at_price = positions.iter().map(|o| o.profit_at_expiry(price)).sum();
             if profit_at_price < max_loss {
                 max_loss = profit_at_price;
                 max_loss_price = price;
@@ -424,9 +697,9 @@ pub fn calculate_profit_bounds_for_strategy(options: &[OptionPosition]) -> Strat
     let max_profit_at_strike = {
         let mut max_profit = Rational64::from_integer(i64::MIN);
         let mut max_profit_price = Rational64::zero();
-        for option in options {
-            let price = option.strike_price;
-            let profit_at_price = options.iter().map(|o| o.profit_at_expiry(price)).sum();
+        for position in positions {
+            let price = position.equivalent_strike_price();
+            let profit_at_price = positions.iter().map(|o| o.profit_at_expiry(price)).sum();
             if profit_at_price > max_profit {
                 max_profit = profit_at_price;
                 max_profit_price = price;
@@ -442,7 +715,7 @@ pub fn calculate_profit_bounds_for_strategy(options: &[OptionPosition]) -> Strat
         ProfitBound::Infinite
     } else if min_gradient < 0 {
         let price = Rational64::zero();
-        let profit_at_zero = options.iter().map(|o| o.profit_at_expiry(price)).sum();
+        let profit_at_zero = positions.iter().map(|o| o.profit_at_expiry(price)).sum();
 
         // profit at zero may not necessarily be the extreme
         if max_loss_at_strike.finite_value().unwrap() <= profit_at_zero {
@@ -461,7 +734,7 @@ pub fn calculate_profit_bounds_for_strategy(options: &[OptionPosition]) -> Strat
         ProfitBound::Infinite
     } else if min_gradient > 0 {
         let price = Rational64::zero();
-        let profit_at_zero = options.iter().map(|o| o.profit_at_expiry(price)).sum();
+        let profit_at_zero = positions.iter().map(|o| o.profit_at_expiry(price)).sum();
 
         // profit at zero may not necessarily be the extreme
         if max_profit_at_strike.finite_value().unwrap() >= profit_at_zero {
@@ -552,8 +825,13 @@ fn calculate_probability_of_expiring_gt_price(
     let stock_price = underlying_price.to_f64()?;
     let expiration_implied_volatility = iv_provider.find_iv_for_expiration_date(expiration_date)?;
 
-    let time = expiration_date.time_to_expiration(now).num_minutes() as f64
-        / Duration::days(365).num_minutes() as f64;
+    let num_minutes: u32 = expiration_date
+        .time_to_expiration(now)
+        .num_minutes()
+        .try_into()
+        .ok()?;
+    let year_minutes: u32 = Duration::days(365).num_minutes().try_into().ok()?;
+    let time = f64::from(num_minutes) / f64::from(year_minutes);
     let vol = expiration_implied_volatility * time.sqrt();
 
     // https://www.ltnielsen.com/wp-content/uploads/Understanding.pdf
@@ -667,7 +945,7 @@ mod tests {
 
     #[test]
     fn test_short_call_profit_at_expiry() {
-        let option = OptionPosition::mock(OptionType::Call, 100, 300, 1);
+        let option = OptionsPosition::mock(OptionType::Call, 100, 300, 1);
         let underlying_price = Rational64::from_integer(101);
         let profit = option.profit_at_expiry(underlying_price);
 
@@ -676,7 +954,7 @@ mod tests {
 
     #[test]
     fn test_short_put_profit_at_expiry() {
-        let option = OptionPosition::mock(OptionType::Put, 100, 300, 1);
+        let option = OptionsPosition::mock(OptionType::Put, 100, 300, 1);
         let underlying_price = Rational64::from_integer(99);
         let profit = option.profit_at_expiry(underlying_price);
 
@@ -685,7 +963,7 @@ mod tests {
 
     #[test]
     fn test_long_put_profit_at_expiry() {
-        let option = OptionPosition::mock(OptionType::Put, 100, -300, 1);
+        let option = OptionsPosition::mock(OptionType::Put, 100, -300, 1);
         let underlying_price = Rational64::from_integer(99);
         let profit = option.profit_at_expiry(underlying_price);
 
@@ -695,8 +973,8 @@ mod tests {
     #[test]
     fn test_calculate_breakevens_for_short_strangle() {
         let options = [
-            OptionPosition::mock(OptionType::Put, 20, 37, 1),
-            OptionPosition::mock(OptionType::Call, 28, 74, 1),
+            OptionsPosition::mock(OptionType::Put, 20, 37, 1),
+            OptionsPosition::mock(OptionType::Call, 28, 74, 1),
         ];
 
         let breakevens = calculate_breakevens_for_strategy(&options);
@@ -721,8 +999,8 @@ mod tests {
     #[test]
     fn test_calculate_breakevens_for_short_call_ratio_spread() {
         let options = [
-            OptionPosition::mock(OptionType::Call, 15, -305, 1),
-            OptionPosition::mock(OptionType::Call, 20, 217, 2),
+            OptionsPosition::mock(OptionType::Call, 15, -305, 1),
+            OptionsPosition::mock(OptionType::Call, 20, 217, 2),
         ];
 
         let breakevens = calculate_breakevens_for_strategy(&options);
@@ -740,7 +1018,7 @@ mod tests {
 
     #[test]
     fn test_calculate_profit_for_long_call() {
-        let options = [OptionPosition::mock(OptionType::Call, 20, -37, 1)];
+        let options = [OptionsPosition::mock(OptionType::Call, 20, -37, 1)];
 
         let profit_bounds = calculate_profit_bounds_for_strategy(&options);
 
@@ -758,7 +1036,7 @@ mod tests {
 
     #[test]
     fn test_calculate_profit_for_long_put() {
-        let options = [OptionPosition::mock(OptionType::Put, 20, -37, 1)];
+        let options = [OptionsPosition::mock(OptionType::Put, 20, -37, 1)];
 
         let profit_bounds = calculate_profit_bounds_for_strategy(&options);
 
@@ -780,8 +1058,8 @@ mod tests {
     #[test]
     fn test_calculate_profit_for_short_strangle() {
         let options = [
-            OptionPosition::mock(OptionType::Put, 20, 37, 1),
-            OptionPosition::mock(OptionType::Call, 28, 74, 1),
+            OptionsPosition::mock(OptionType::Put, 20, 37, 1),
+            OptionsPosition::mock(OptionType::Call, 28, 74, 1),
         ];
 
         let profit_bounds = calculate_profit_bounds_for_strategy(&options);
@@ -801,8 +1079,8 @@ mod tests {
     #[test]
     fn test_calculate_profit_for_long_strangle() {
         let options = [
-            OptionPosition::mock(OptionType::Put, 20, -37, 1),
-            OptionPosition::mock(OptionType::Call, 28, -74, 1),
+            OptionsPosition::mock(OptionType::Put, 20, -37, 1),
+            OptionsPosition::mock(OptionType::Call, 28, -74, 1),
         ];
 
         let profit_bounds = calculate_profit_bounds_for_strategy(&options);
@@ -822,8 +1100,8 @@ mod tests {
     #[test]
     fn test_calculate_profit_for_short_call_ratio_spread() {
         let options = [
-            OptionPosition::mock(OptionType::Call, 15, -305, 1),
-            OptionPosition::mock(OptionType::Call, 20, 217, 2),
+            OptionsPosition::mock(OptionType::Call, 15, -305, 1),
+            OptionsPosition::mock(OptionType::Call, 20, 217, 2),
         ];
 
         let profit_bounds = calculate_profit_bounds_for_strategy(&options);
@@ -843,8 +1121,8 @@ mod tests {
     #[test]
     fn test_calculate_profit_for_long_put_ratio_spread() {
         let options = [
-            OptionPosition::mock(OptionType::Put, 20, 305, 1),
-            OptionPosition::mock(OptionType::Put, 15, -217, 2),
+            OptionsPosition::mock(OptionType::Put, 20, 305, 1),
+            OptionsPosition::mock(OptionType::Put, 15, -217, 2),
         ];
 
         let profit_bounds = calculate_profit_bounds_for_strategy(&options);
@@ -866,6 +1144,65 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_calculate_profit_for_covered_call() {
+        let positions = [
+            SharesPosition::mock(-11, 200),
+            OptionsPosition::mock(OptionType::Call, 20, 30, 2),
+        ];
+
+        let profit_bounds = calculate_profit_bounds_for_strategy(&positions);
+
+        let max_loss = -11 * 200 + 30 * 2;
+        let max_profit = (20 - 11) * 200 + 30 * 2;
+        assert_eq!(
+            profit_bounds,
+            StrategyProfitBounds {
+                max_loss: Some(ProfitBound::Finite {
+                    value: Rational64::from_integer(max_loss),
+                    price: Rational64::from_integer(0)
+                }),
+                max_profit: Some(ProfitBound::Finite {
+                    value: Rational64::from_integer(max_profit),
+                    price: Rational64::from_integer(20)
+                }),
+            }
+        );
+    }
+
+    #[test]
+    fn test_calculate_breakevens_for_shares() {
+        let positions = [
+            SharesPosition::mock(-11, 150), // long
+            SharesPosition::mock(19, 50),   // short
+        ];
+        let breakevens = calculate_breakevens_for_strategy(&positions);
+        assert_eq!(
+            breakevens,
+            StrategyBreakevens {
+                breakevens: vec![Breakeven {
+                    price: Rational64::from_integer(7),
+                    is_ascending: true,
+                }]
+            }
+        );
+    }
+
+    #[test]
+    fn test_calculate_breakevens_for_leap() {
+        let positions = [OptionsPosition::mock(OptionType::Call, 1, -30, 1)];
+        let breakevens = calculate_breakevens_for_strategy(&positions);
+        assert_eq!(
+            breakevens,
+            StrategyBreakevens {
+                breakevens: vec![Breakeven {
+                    price: Rational64::new(13, 10),
+                    is_ascending: true,
+                }]
+            }
+        );
+    }
+
     #[test]
     fn test_calculate_pop_no_loss() {
         struct IVProvider;
@@ -892,10 +1229,10 @@ mod tests {
     #[test]
     fn test_calculate_pop_multiple_breakevens() {
         let option_positions = [
-            OptionPosition::mock(OptionType::Put, 20, -55, 2),
-            OptionPosition::mock(OptionType::Put, 30, 277, 1),
-            OptionPosition::mock(OptionType::Call, 60, -823, 1),
-            OptionPosition::mock(OptionType::Call, 85, 456, 2),
+            OptionsPosition::mock(OptionType::Put, 20, -55, 2),
+            OptionsPosition::mock(OptionType::Put, 30, 277, 1),
+            OptionsPosition::mock(OptionType::Call, 60, -823, 1),
+            OptionsPosition::mock(OptionType::Call, 85, 456, 2),
         ];
 
         let breakevens = calculate_breakevens_for_strategy(&option_positions);
